@@ -17,18 +17,20 @@ for k in backup:
 
 class EnginePatcher(object):
     """This class implements the context manager interface for syntax patching.
-
-    :param patches: a mapping of members to be patched
-    :type patches: `dict`
     """
     def __init__(self, patches):
+        """Initialize a :class:`Patcher` object.
+
+        :param patches: a mapping of members to be patched
+        :type patches: `dict`
+        """
         self._backup = {}
         self._patches = patches
 
     def __enter__(self):
-        for k in self._patches:
+        for k in backup:
             self._backup[k] = getattr(mosql.util, k)
-            setattr(mosql.util, k, self._patches[k])
+            setattr(mosql.util, k, self._patches.get(k, backup[k]))
         return self._backup
 
     def __exit__(self, exc_type, exc_val, exc_tb):
